@@ -30,19 +30,19 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
 
+	pkgnet "knative.dev/networking/pkg/apis/networking"
 	fakekubeclient "knative.dev/pkg/client/injection/kube/client/fake"
 	fakeendpointsinformer "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
 	fakeserviceinformer "knative.dev/pkg/client/injection/kube/informers/core/v1/service/fake"
-	"knative.dev/pkg/controller"
 	"knative.dev/pkg/network"
 	"knative.dev/pkg/ptr"
 	rtesting "knative.dev/pkg/reconciler/testing"
 	activatortest "knative.dev/serving/pkg/activator/testing"
-	"knative.dev/serving/pkg/apis/networking"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	fakeservingclient "knative.dev/serving/pkg/client/injection/client/fake"
 	fakerevisioninformer "knative.dev/serving/pkg/client/injection/informers/serving/v1/revision/fake"
+	"knative.dev/serving/pkg/networking"
 	"knative.dev/serving/pkg/queue"
 
 	. "knative.dev/pkg/logging/testing"
@@ -53,15 +53,15 @@ const (
 	testRevision  = "test-revision"
 
 	probeFreq     = 50 * time.Millisecond
-	updateTimeout = 6 * probeFreq
+	updateTimeout = 16 * probeFreq
 )
 
 // revisionCC1 - creates a revision with concurrency == 1.
-func revisionCC1(revID types.NamespacedName, protocol networking.ProtocolType) *v1.Revision {
+func revisionCC1(revID types.NamespacedName, protocol pkgnet.ProtocolType) *v1.Revision {
 	return revision(revID, protocol, 1)
 }
 
-func revision(revID types.NamespacedName, protocol networking.ProtocolType, cc int64) *v1.Revision {
+func revision(revID types.NamespacedName, protocol pkgnet.ProtocolType, cc int64) *v1.Revision {
 	return &v1.Revision{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: revID.Namespace,
@@ -97,7 +97,7 @@ func privateSKSService(revID types.NamespacedName, clusterIP string, ports []cor
 	}
 }
 
-func waitForRevisionBackedMananger(t *testing.T, rbm *revisionBackendsManager) {
+func waitForRevisionBackendManager(t *testing.T, rbm *revisionBackendsManager) {
 	timeout := time.After(updateTimeout)
 	for {
 		select {
@@ -118,7 +118,7 @@ func TestRevisionWatcher(t *testing.T) {
 	for _, tc := range []struct {
 		name                  string
 		dests                 dests
-		protocol              networking.ProtocolType
+		protocol              pkgnet.ProtocolType
 		clusterPort           corev1.ServicePort
 		clusterIP             string
 		expectUpdates         []revisionDestsUpdate
@@ -158,7 +158,7 @@ func TestRevisionWatcher(t *testing.T) {
 	}, {
 		name:     "single http2 podIP",
 		dests:    dests{ready: sets.NewString("128.0.0.1:1234")},
-		protocol: networking.ProtocolH2C,
+		protocol: pkgnet.ProtocolH2C,
 		clusterPort: corev1.ServicePort{
 			Name: "http2",
 			Port: 1234,
@@ -177,7 +177,7 @@ func TestRevisionWatcher(t *testing.T) {
 	}, {
 		name:     "single http2 clusterIP",
 		dests:    dests{ready: sets.NewString("128.0.0.1:1234"), notReady: sets.NewString("128.0.0.2:1234")},
-		protocol: networking.ProtocolH2C,
+		protocol: pkgnet.ProtocolH2C,
 		clusterPort: corev1.ServicePort{
 			Name: "http2",
 			Port: 1234,
@@ -389,9 +389,9 @@ func TestRevisionWatcher(t *testing.T) {
 			// This gets closed up by revisionWatcher
 			destsCh := make(chan dests)
 
-			// Default for protocol is http1
+			// Default for protocol is HTTP1
 			if tc.protocol == "" {
-				tc.protocol = networking.ProtocolHTTP1
+				tc.protocol = pkgnet.ProtocolHTTP1
 			}
 
 			fake := fakekubeclient.Get(ctx)
@@ -400,11 +400,11 @@ func TestRevisionWatcher(t *testing.T) {
 			revID := types.NamespacedName{Namespace: testNamespace, Name: testRevision}
 			if tc.clusterIP != "" {
 				svc := privateSKSService(revID, tc.clusterIP, []corev1.ServicePort{tc.clusterPort})
-				fake.CoreV1().Services(svc.Namespace).Create(svc)
+				fake.CoreV1().Services(svc.Namespace).Create(ctx, svc, metav1.CreateOptions{})
 				informer.Informer().GetIndexer().Add(svc)
 			}
 
-			waitInformers, err := controller.RunInformers(ctx.Done(), informer.Informer())
+			waitInformers, err := rtesting.RunAndSyncInformers(ctx, informer.Informer())
 			if err != nil {
 				t.Fatal("Failed to start informers:", err)
 			}
@@ -441,7 +441,7 @@ func TestRevisionWatcher(t *testing.T) {
 				case update := <-updateCh:
 					updates = append(updates, update)
 				case <-time.After(updateTimeout):
-					t.Error("Timed out waiting for update event")
+					t.Fatal("Timed out waiting for update event")
 				}
 			}
 			if got, want := rw.podsAddressable, !tc.noPodAddressability; got != want {
@@ -460,7 +460,7 @@ func TestRevisionWatcher(t *testing.T) {
 			}
 
 			if got, want := updates, tc.expectUpdates; !cmp.Equal(got, want, cmpopts.EquateEmpty()) {
-				t.Errorf("revisionDests updates = %v, want: %v, diff (-want, +got):\n %s", got, want, cmp.Diff(want, got))
+				t.Errorf("revisionDests updates = %v, want: %v, diff(-want, +got):\n%s", got, want, cmp.Diff(want, got))
 			}
 		})
 	}
@@ -480,7 +480,7 @@ func assertChClosed(t *testing.T, ch chan struct{}) {
 	}
 }
 
-func epSubset(port int32, portName string, ips []string, notReadyIps []string) *corev1.EndpointSubset {
+func epSubset(port int32, portName string, ips, notReadyIps []string) *corev1.EndpointSubset {
 	ss := &corev1.EndpointSubset{
 		Ports: []corev1.EndpointPort{{
 			Name: portName,
@@ -500,7 +500,7 @@ func ep(revL string, port int32, portName string, ips ...string) *corev1.Endpoin
 	return epNotReady(revL, port, portName, ips, nil)
 }
 
-func epNotReady(revL string, port int32, portName string, readyIps []string, notReadyIps []string) *corev1.Endpoints {
+func epNotReady(revL string, port int32, portName string, readyIps, notReadyIps []string) *corev1.Endpoints {
 	return &corev1.Endpoints{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: revL + "-ep",
@@ -528,7 +528,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "add slow healthy",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, "129.0.0.1",
@@ -556,7 +556,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "add slow ready http2",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http2", "128.0.0.1")},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolH2C),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolH2C),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, "129.0.0.1",
@@ -587,8 +587,8 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 			ep("test-revision2", 1235, "http", "128.1.0.2"),
 		},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: "test-revision1"}, networking.ProtocolHTTP1),
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: "test-revision2"}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: "test-revision1"}, pkgnet.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: "test-revision2"}, pkgnet.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: "test-revision1"}, "129.0.0.1",
@@ -613,7 +613,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "no pod addressability",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, "129.0.0.1",
@@ -639,7 +639,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "unhealthy",
 		endpointsArr: []*corev1.Endpoints{ep(testRevision, 1234, "http", "128.0.0.1")},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, "129.0.0.1",
@@ -659,7 +659,7 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 		name:         "unready pod successfully probed",
 		endpointsArr: []*corev1.Endpoints{epNotReady(testRevision, 1234, "http", nil, []string{"128.0.0.1"})},
 		revisions: []*v1.Revision{
-			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1),
+			revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1),
 		},
 		services: []*corev1.Service{
 			privateSKSService(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, "129.0.0.1",
@@ -696,16 +696,16 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 
 			// Add the revision we're testing.
 			for _, rev := range tc.revisions {
-				fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+				fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 				revisions.Informer().GetIndexer().Add(rev)
 			}
 
 			for _, svc := range tc.services {
-				fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+				fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
 				serviceInformer.Informer().GetIndexer().Add(svc)
 			}
 
-			waitInformers, err := controller.RunInformers(ctx.Done(), endpointsInformer.Informer())
+			waitInformers, err := rtesting.RunAndSyncInformers(ctx, endpointsInformer.Informer())
 			if err != nil {
 				t.Fatal("Failed to start informers:", err)
 			}
@@ -714,11 +714,11 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 			defer func() {
 				cancel()
 				waitInformers()
-				waitForRevisionBackedMananger(t, rbm)
+				waitForRevisionBackendManager(t, rbm)
 			}()
 
 			for _, ep := range tc.endpointsArr {
-				fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
+				fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ctx, ep, metav1.CreateOptions{})
 				endpointsInformer.Informer().GetIndexer().Add(ep)
 			}
 
@@ -746,6 +746,154 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 	}
 }
 
+func emptyDests() dests {
+	return dests{
+		ready:    sets.NewString(),
+		notReady: sets.NewString(),
+	}
+}
+
+func TestCheckDestsReadyToNotReady(t *testing.T) {
+	// This test verifies the edge behaviour when a pod
+	// previously in the ready sed moved to non ready set
+	// and now must be re-probed.
+	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
+
+	svc := privateSKSService(
+		types.NamespacedName{Namespace: testNamespace, Name: testRevision},
+		"129.0.0.1",
+		[]corev1.ServicePort{{Name: "http", Port: 1234}},
+	)
+	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
+	si := fakeserviceinformer.Get(ctx)
+	si.Informer().GetIndexer().Add(svc)
+
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, si.Informer())
+	if err != nil {
+		t.Fatal("Failed to start informers:", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
+	fakeRT := activatortest.FakeRoundTripper{
+		ExpectHost: testRevision,
+		ProbeHostResponses: map[string][]activatortest.FakeResponse{
+			"10.10.1.1": {{
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				// Finally!
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"10.10.1.2": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"10.10.1.3": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
+	}
+
+	// Make it buffered,so that we can make the test linear.
+	uCh := make(chan revisionDestsUpdate, 1)
+	dCh := make(chan struct{})
+	rw := &revisionWatcher{
+		clusterIPHealthy: true,
+		podsAddressable:  true,
+		rev:              types.NamespacedName{Namespace: testNamespace, Name: testRevision},
+		updateCh:         uCh,
+		serviceLister:    si.Lister(),
+		logger:           TestLogger(t),
+		stopCh:           dCh,
+		transport:        network.RoundTripperFunc(fakeRT.RT),
+	}
+	// Initial state. Both are ready.
+	cur := dests{
+		ready:    sets.NewString("10.10.1.3", "10.10.1.2"),
+		notReady: sets.NewString("10.10.1.1"),
+	}
+	rw.checkDests(cur, emptyDests())
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 2; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+	// Repeating should be a no-op.
+	rw.checkDests(cur, emptyDests())
+	select {
+	case u := <-uCh:
+		t.Fatal("Unexpected update", u)
+	default:
+		// Success.
+	}
+
+	// Now swing the ready to the non ready, and it should fail the probe.
+	prev := cur
+
+	cur = dests{
+		ready:    sets.NewString("10.10.1.2"),
+		notReady: sets.NewString("10.10.1.1", "10.10.1.3"),
+	}
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		// The 10.10.1.3 should be not ready now.
+		if got, want := len(u.Dests), 1; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+
+	// Now just re-probe the same and it should succeed (see the RT setup above).
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 2; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+
+	// Timer update.
+	prev.ready.Delete("10.10.1.3")
+	// The last one should finally succeed.
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 3; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+}
+
 func TestCheckDests(t *testing.T) {
 	// This test covers some edge cases in `checkDests` which are next to impossible to
 	// test via tests above.
@@ -757,11 +905,11 @@ func TestCheckDests(t *testing.T) {
 		"129.0.0.1",
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
 	)
-	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
 	si := fakeserviceinformer.Get(ctx)
 	si.Informer().GetIndexer().Add(svc)
 
-	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, si.Informer())
 	if err != nil {
 		t.Fatal("Failed to start informers:", err)
 	}
@@ -782,7 +930,10 @@ func TestCheckDests(t *testing.T) {
 		logger:           TestLogger(t),
 		stopCh:           dCh,
 	}
-	rw.checkDests(dests{ready: sets.NewString("10.1.1.5"), notReady: sets.NewString("10.1.1.6")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.1.1.5"),
+		notReady: sets.NewString("10.1.1.6"),
+	}, emptyDests())
 	select {
 	case <-uCh:
 		// Success.
@@ -791,7 +942,10 @@ func TestCheckDests(t *testing.T) {
 	}
 
 	close(dCh)
-	rw.checkDests(dests{ready: sets.NewString("10.1.1.5"), notReady: sets.NewString("10.1.1.6")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.1.1.5"),
+		notReady: sets.NewString("10.1.1.6"),
+	}, emptyDests())
 	select {
 	case <-uCh:
 		t.Error("Expected no update but got one")
@@ -811,11 +965,11 @@ func TestCheckDestsSwinging(t *testing.T) {
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
 	)
 
-	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
 	si := fakeserviceinformer.Get(ctx)
 	si.Informer().GetIndexer().Add(svc)
 
-	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, si.Informer())
 	if err != nil {
 		t.Fatal("Failed to start informers:", err)
 	}
@@ -876,8 +1030,9 @@ func TestCheckDestsSwinging(t *testing.T) {
 		podsAddressable: true,
 		transport:       network.RoundTripperFunc(fakeRT.RT),
 	}
+
 	// First not ready, second good, clusterIP: not ready.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	want := revisionDestsUpdate{
 		Rev:           types.NamespacedName{Namespace: testNamespace, Name: testRevision},
 		ClusterIPDest: "",
@@ -894,7 +1049,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Second gone, first becomes ready, clusterIP still not ready.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.1:1234")
@@ -906,7 +1061,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Second is back, but not healthy yet.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		// No update should be sent out, since there's only healthy pod, same as above.
@@ -915,7 +1070,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// All pods are happy now.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234", "10.0.0.1:1234")
@@ -927,7 +1082,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Make sure we do not send out redundant updates.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		t.Errorf("Expected no update, but got %#v", got)
@@ -936,7 +1091,10 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Add a notReady pod, but it's not ready. No update should be sent.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"), notReady: sets.NewString("10.0.0.4:1234")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"),
+		notReady: sets.NewString("10.0.0.4:1234"),
+	}, emptyDests())
 	select {
 	case got := <-uCh:
 		t.Errorf("Expected no update, but got %#v", got)
@@ -945,7 +1103,10 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// The notReady pod is now ready!
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"), notReady: sets.NewString("10.0.0.4:1234")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"),
+		notReady: sets.NewString("10.0.0.4:1234"),
+	}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.1:1234", "10.0.0.2:1234", "10.0.0.4:1234")
@@ -957,7 +1118,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Swing to a different pods.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.3:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.3:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234", "10.0.0.3:1234")
@@ -969,7 +1130,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Scale down by 1.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234")
@@ -989,20 +1150,20 @@ func TestRevisionDeleted(t *testing.T) {
 		"129.0.0.1",
 		[]corev1.ServicePort{{Name: "http", Port: 1234}},
 	)
-	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
 	si := fakeserviceinformer.Get(ctx)
 	si.Informer().GetIndexer().Add(svc)
 
 	ei := fakeendpointsinformer.Get(ctx)
 	ep := ep(testRevision, 1234, "http", "128.0.0.1")
-	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
-	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ctx, ep, metav1.CreateOptions{})
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, ei.Informer())
 	if err != nil {
 		t.Fatal("Failed to start informers:", err)
 	}
 
-	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	ri := fakerevisioninformer.Get(ctx)
 	ri.Informer().GetIndexer().Add(rev)
 
@@ -1011,7 +1172,7 @@ func TestRevisionDeleted(t *testing.T) {
 	defer func() {
 		cancel()
 		waitInformers()
-		waitForRevisionBackedMananger(t, rbm)
+		waitForRevisionBackendManager(t, rbm)
 	}()
 
 	// Make some movements.
@@ -1022,7 +1183,7 @@ func TestRevisionDeleted(t *testing.T) {
 		t.Error("Timedout waiting for initial response")
 	}
 	// Now delete the endpoints.
-	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Delete(ep.Name, &metav1.DeleteOptions{})
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Delete(ctx, ep.Name, metav1.DeleteOptions{})
 	select {
 	case r := <-rbm.updates():
 		t.Errorf("Unexpected update: %#v", r)
@@ -1037,14 +1198,14 @@ func TestServiceDoesNotExist(t *testing.T) {
 
 	ei := fakeendpointsinformer.Get(ctx)
 	eps := ep(testRevision, 1234, "http", "128.0.0.1")
-	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
-	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ctx, eps, metav1.CreateOptions{})
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, ei.Informer())
 	if err != nil {
 		t.Fatal("Failed to start informers:", err)
 	}
 
-	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	ri := fakerevisioninformer.Get(ctx)
 	ri.Informer().GetIndexer().Add(rev)
 
@@ -1067,7 +1228,7 @@ func TestServiceDoesNotExist(t *testing.T) {
 	defer func() {
 		cancel()
 		waitInformers()
-		waitForRevisionBackedMananger(t, rbm)
+		waitForRevisionBackendManager(t, rbm)
 	}()
 
 	// Make some movements to generate a checkDests call.
@@ -1077,7 +1238,7 @@ func TestServiceDoesNotExist(t *testing.T) {
 		// We can't probe endpoints (see RT above) and we can't get to probe
 		// cluster IP. But if the service is accessible then we will and probing will
 		// succeed since RT has no rules for that.
-		t.Errorf("Unexpected update, should have had none: %v", x)
+		t.Error("Unexpected update, should have had none:", x)
 	case <-time.After(updateTimeout):
 	}
 }
@@ -1088,13 +1249,13 @@ func TestServiceMoreThanOne(t *testing.T) {
 
 	ei := fakeendpointsinformer.Get(ctx)
 	eps := ep(testRevision, 1234, "http", "128.0.0.1")
-	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
-	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
+	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ctx, eps, metav1.CreateOptions{})
+	waitInformers, err := rtesting.RunAndSyncInformers(ctx, ei.Informer())
 	if err != nil {
 		t.Fatal("Failed to start informers:", err)
 	}
-	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
-	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
+	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, pkgnet.ProtocolHTTP1)
+	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(ctx, rev, metav1.CreateOptions{})
 	ri := fakerevisioninformer.Get(ctx)
 	ri.Informer().GetIndexer().Add(rev)
 
@@ -1106,8 +1267,8 @@ func TestServiceMoreThanOne(t *testing.T) {
 			[]corev1.ServicePort{{Name: "http", Port: 1234}},
 		)
 		// Modify the name so both can be created.
-		svc.Name = svc.Name + num
-		fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+		svc.Name += num
+		fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(ctx, svc, metav1.CreateOptions{})
 		si := fakeserviceinformer.Get(ctx)
 		si.Informer().GetIndexer().Add(svc)
 	}
@@ -1131,7 +1292,7 @@ func TestServiceMoreThanOne(t *testing.T) {
 	defer func() {
 		cancel()
 		waitInformers()
-		waitForRevisionBackedMananger(t, rbm)
+		waitForRevisionBackendManager(t, rbm)
 	}()
 
 	ei.Informer().GetIndexer().Add(eps)
