@@ -28,7 +28,7 @@ import (
 	pkgmetrics "knative.dev/pkg/metrics"
 	"knative.dev/pkg/ptr"
 	pkgreconciler "knative.dev/pkg/reconciler"
-	pav1alpha1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
+	autoscalingv1alpha1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
 	"knative.dev/serving/pkg/apis/serving"
 	"knative.dev/serving/pkg/autoscaler/scaling"
 	pareconciler "knative.dev/serving/pkg/client/injection/reconciler/autoscaling/v1alpha1/podautoscaler"
@@ -72,7 +72,8 @@ type Reconciler struct {
 // Check that our Reconciler implements pareconciler.Interface
 var _ pareconciler.Interface = (*Reconciler)(nil)
 
-func (c *Reconciler) ReconcileKind(ctx context.Context, pa *pav1alpha1.PodAutoscaler) pkgreconciler.Event {
+// ReconcileKind implements Interface.ReconcileKind.
+func (c *Reconciler) ReconcileKind(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler) pkgreconciler.Event {
 	logger := logging.FromContext(ctx)
 
 	// We need the SKS object in order to optimize scale to zero
@@ -91,7 +92,8 @@ func (c *Reconciler) ReconcileKind(ctx context.Context, pa *pav1alpha1.PodAutosc
 			return fmt.Errorf("error reconciling SKS: %w", err)
 		}
 		pa.Status.MarkSKSNotReady(noPrivateServiceName) // In both cases this is true.
-		return computeStatus(ctx, pa, podCounts{want: scaleUnknown}, logger)
+		computeStatus(ctx, pa, podCounts{want: scaleUnknown}, logger)
+		return nil
 	}
 
 	pa.Status.MetricsServiceName = sks.Status.PrivateServiceName
@@ -162,11 +164,12 @@ func (c *Reconciler) ReconcileKind(ctx context.Context, pa *pav1alpha1.PodAutosc
 		terminating: terminating,
 	}
 	logger.Infof("Observed pod counts=%#v", pc)
-	return computeStatus(ctx, pa, pc, logger)
+	computeStatus(ctx, pa, pc, logger)
+	return nil
 }
 
-func (c *Reconciler) reconcileDecider(ctx context.Context, pa *pav1alpha1.PodAutoscaler) (*scaling.Decider, error) {
-	desiredDecider := resources.MakeDecider(ctx, pa, config.FromContext(ctx).Autoscaler)
+func (c *Reconciler) reconcileDecider(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler) (*scaling.Decider, error) {
+	desiredDecider := resources.MakeDecider(pa, config.FromContext(ctx).Autoscaler)
 	decider, err := c.deciders.Get(ctx, desiredDecider.Namespace, desiredDecider.Name)
 	if errors.IsNotFound(err) {
 		decider, err = c.deciders.Create(ctx, desiredDecider)
@@ -188,27 +191,19 @@ func (c *Reconciler) reconcileDecider(ctx context.Context, pa *pav1alpha1.PodAut
 	return decider, nil
 }
 
-func computeStatus(ctx context.Context, pa *pav1alpha1.PodAutoscaler, pc podCounts, logger *zap.SugaredLogger) error {
+func computeStatus(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler, pc podCounts, logger *zap.SugaredLogger) {
 	pa.Status.DesiredScale, pa.Status.ActualScale = ptr.Int32(int32(pc.want)), ptr.Int32(int32(pc.ready))
 
-	if err := reportMetrics(pa, pc); err != nil {
-		return fmt.Errorf("error reporting metrics: %w", err)
-	}
-
+	reportMetrics(pa, pc)
 	computeActiveCondition(ctx, pa, pc)
 	logger.Debugf("PA Status after reconcile: %#v", pa.Status.Status)
-
-	return nil
 }
 
-func reportMetrics(pa *pav1alpha1.PodAutoscaler, pc podCounts) error {
+func reportMetrics(pa *autoscalingv1alpha1.PodAutoscaler, pc podCounts) {
 	serviceLabel := pa.Labels[serving.ServiceLabelKey] // This might be empty.
 	configLabel := pa.Labels[serving.ConfigurationLabelKey]
 
-	ctx, err := metrics.RevisionContext(pa.Namespace, serviceLabel, configLabel, pa.Name)
-	if err != nil {
-		return err
-	}
+	ctx := metrics.RevisionContext(pa.Namespace, serviceLabel, configLabel, pa.Name)
 
 	stats := []stats.Measurement{
 		actualPodCountM.M(int64(pc.ready)), notReadyPodCountM.M(int64(pc.notReady)),
@@ -219,7 +214,6 @@ func reportMetrics(pa *pav1alpha1.PodAutoscaler, pc podCounts) error {
 		stats = append(stats, requestedPodCountM.M(int64(pc.want)))
 	}
 	pkgmetrics.RecordBatch(ctx, stats...)
-	return nil
 }
 
 // computeActiveCondition updates the status of a PA given the current scale (got), desired scale (want)
@@ -237,13 +231,9 @@ func reportMetrics(pa *pav1alpha1.PodAutoscaler, pc podCounts) error {
 //    | -1   | >= min | 0     | active     | inactive   | <-- this case technically is impossible.
 //    | -1   | >= min | >0    | activating | active     |
 //    | -1   | >= min | >0    | active     | active     |
-func computeActiveCondition(ctx context.Context, pa *pav1alpha1.PodAutoscaler, pc podCounts) {
+func computeActiveCondition(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler, pc podCounts) {
 	minReady := activeThreshold(ctx, pa)
-	// In pre-0.17 we could have scaled down normally without ever setting ScaleTargetInitialized.
-	// In this case we'll be in the NoTraffic/inactive state.
-	// TODO(taragu): remove after 0.19
-	alreadyScaledDownSuccessfully := minReady > 0 && pa.Status.GetCondition(pav1alpha1.PodAutoscalerConditionActive).GetReason() == noTrafficReason
-	if (pc.ready >= minReady || alreadyScaledDownSuccessfully) && pa.Status.ServiceName != "" {
+	if pc.ready >= minReady && pa.Status.ServiceName != "" {
 		pa.Status.MarkScaleTargetInitialized()
 	}
 
@@ -280,7 +270,7 @@ func computeActiveCondition(ctx context.Context, pa *pav1alpha1.PodAutoscaler, p
 }
 
 // activeThreshold returns the scale required for the pa to be marked Active
-func activeThreshold(ctx context.Context, pa *pav1alpha1.PodAutoscaler) int {
+func activeThreshold(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler) int {
 	asConfig := config.FromContext(ctx).Autoscaler
 	min, _ := pa.ScaleBounds(asConfig)
 	if !pa.Status.IsScaleTargetInitialized() {
@@ -292,7 +282,7 @@ func activeThreshold(ctx context.Context, pa *pav1alpha1.PodAutoscaler) int {
 
 // resolveScrapeTarget returns metric service name to be scraped based on TBC configuration
 // TBC == -1 => activator in path, don't scrape the service
-func resolveScrapeTarget(ctx context.Context, pa *pav1alpha1.PodAutoscaler) string {
+func resolveScrapeTarget(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler) string {
 	tbc := resolveTBC(ctx, pa)
 	if tbc == -1 {
 		return ""
@@ -301,7 +291,7 @@ func resolveScrapeTarget(ctx context.Context, pa *pav1alpha1.PodAutoscaler) stri
 	return pa.Status.MetricsServiceName
 }
 
-func resolveTBC(ctx context.Context, pa *pav1alpha1.PodAutoscaler) float64 {
+func resolveTBC(ctx context.Context, pa *autoscalingv1alpha1.PodAutoscaler) float64 {
 	if v, ok := pa.TargetBC(); ok {
 		return v
 	}
