@@ -22,6 +22,7 @@ import (
 	"sort"
 	"strconv"
 	"testing"
+	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"go.uber.org/zap/zapcore"
@@ -36,15 +37,19 @@ import (
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/logging"
 	"knative.dev/pkg/metrics"
-	_ "knative.dev/pkg/metrics/testing"
 	"knative.dev/pkg/ptr"
 	"knative.dev/pkg/system"
-	_ "knative.dev/pkg/system/testing"
 	tracingconfig "knative.dev/pkg/tracing/config"
+	apicfg "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
-	asconfig "knative.dev/serving/pkg/autoscaler/config"
+	"knative.dev/serving/pkg/autoscaler/config/autoscalerconfig"
 	"knative.dev/serving/pkg/deployment"
+	"knative.dev/serving/pkg/queue"
+	"knative.dev/serving/pkg/reconciler/revision/config"
+
+	_ "knative.dev/pkg/metrics/testing"
+	_ "knative.dev/pkg/system/testing"
 )
 
 var (
@@ -63,14 +68,29 @@ var (
 	}}
 
 	// The default CM values.
-	logConfig        logging.Config
-	traceConfig      tracingconfig.Config
-	obsConfig        metrics.ObservabilityConfig
-	deploymentConfig deployment.Config
-	asConfig         = asconfig.Config{
+	asConfig = autoscalerconfig.Config{
 		InitialScale:          1,
 		AllowZeroInitialScale: false,
 	}
+	deploymentConfig = deployment.Config{
+		ProgressDeadline: 5678 * time.Second,
+	}
+	logConfig   logging.Config
+	obsConfig   metrics.ObservabilityConfig
+	traceConfig tracingconfig.Config
+	defaults, _ = apicfg.NewDefaultsConfigFromMap(nil)
+	revCfg      = config.Config{
+		Config: &apicfg.Config{
+			Autoscaler: &asConfig,
+			Defaults:   defaults,
+			Features:   &apicfg.Features{},
+		},
+		Deployment:    &deploymentConfig,
+		Logging:       &logConfig,
+		Network:       &network.Config{},
+		Observability: &obsConfig,
+		Tracing:       &traceConfig,
+	}
 )
 
 const testProbeJSONTemplate = `{"tcpSocket":{"port":%d,"host":"127.0.0.1"}}`
@@ -83,9 +103,13 @@ func TestMakeQueueContainer(t *testing.T) {
 		nc   network.Config
 		oc   metrics.ObservabilityConfig
 		dc   deployment.Config
+		fc   apicfg.Features
 		want corev1.Container
 	}{{
 		name: "autoscaler single",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
 			withContainers(containers),
 			withContainerConcurrency(1)),
@@ -106,11 +130,13 @@ func TestMakeQueueContainer(t *testing.T) {
 				}},
 			}})),
 		dc: deployment.Config{
+			ProgressDeadline:  5678 * time.Second,
 			QueueSidecarImage: "alpine",
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Image = "alpine"
 			c.Ports = append(queueNonServingPorts, queueHTTP2Port)
+			c.ReadinessProbe.Handler.HTTPGet.Port.IntVal = queueHTTP2Port.ContainerPort
 			c.Env = env(map[string]string{
 				"USER_PORT":          "1955",
 				"QUEUE_SERVING_PORT": "8013",
@@ -118,6 +144,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		}),
 	}, {
 		name: "service name in labels",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
 			withContainers(containers),
 			func(revision *v1.Revision) {
@@ -132,6 +161,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		}),
 	}, {
 		name: "config owner as env var, zero concurrency",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("blah", "baz",
 			withContainers(containers),
 			withContainerConcurrency(0),
@@ -154,6 +186,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		}),
 	}, {
 		name: "logging configuration as env var",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("this", "log",
 			withContainers(containers)),
 		lc: logging.Config{
@@ -172,6 +207,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		}),
 	}, {
 		name: "container concurrency 10",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
 			withContainers(containers),
 			withContainerConcurrency(10)),
@@ -184,6 +222,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		name: "request log configuration as env var",
 		rev: revision("bar", "foo",
 			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{
 			RequestLogTemplate:    "test template",
 			EnableProbeRequestLog: true,
@@ -198,6 +239,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		name: "disabled request log configuration as env var",
 		rev: revision("bar", "foo",
 			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{
 			RequestLogTemplate:    "test template",
 			EnableProbeRequestLog: false,
@@ -214,6 +258,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		name: "request metrics backend as env var",
 		rev: revision("bar", "foo",
 			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{
 			RequestMetricsBackend: "prometheus",
 		},
@@ -226,6 +273,9 @@ func TestMakeQueueContainer(t *testing.T) {
 		name: "enable profiling",
 		rev: revision("bar", "foo",
 			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{EnableProfiling: true},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
@@ -235,11 +285,18 @@ func TestMakeQueueContainer(t *testing.T) {
 		}),
 	}, {
 		name: "custom TimeoutSeconds",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainers(containers)),
+			withContainers(containers),
+			func(revision *v1.Revision) {
+				revision.Spec.TimeoutSeconds = ptr.Int64(99)
+			},
+		),
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"REVISION_TIMEOUT_SECONDS": "45",
+				"REVISION_TIMEOUT_SECONDS": "99",
 			})
 		}),
 	}, {
@@ -247,6 +304,7 @@ func TestMakeQueueContainer(t *testing.T) {
 		rev: revision("bar", "foo",
 			withContainers(containers)),
 		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
 			QueueSidecarCPURequest: &deployment.QueueSidecarCPURequestDefault,
 		},
 		want: queueContainer(func(c *corev1.Container) {
@@ -264,6 +322,7 @@ func TestMakeQueueContainer(t *testing.T) {
 			QueueSidecarCPURequest:              resourcePtr(resource.MustParse("123m")),
 			QueueSidecarEphemeralStorageRequest: resourcePtr(resource.MustParse("456M")),
 			QueueSidecarMemoryLimit:             resourcePtr(resource.MustParse("789m")),
+			ProgressDeadline:                    5678 * time.Second,
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{})
@@ -275,6 +334,38 @@ func TestMakeQueueContainer(t *testing.T) {
 				corev1.ResourceMemory: resource.MustParse("789m"),
 			}
 		}),
+	}, {
+		name: "collector address as env var",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		oc: metrics.ObservabilityConfig{
+			RequestMetricsBackend:   "opencensus",
+			MetricsCollectorAddress: "otel:55678",
+		},
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"SERVING_REQUEST_METRICS_BACKEND": "opencensus",
+				"METRICS_COLLECTOR_ADDRESS":       "otel:55678",
+			})
+		}),
+	}, {
+		name: "HTTP2 autodetection enabled",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		fc: apicfg.Features{
+			AutoDetectHTTP2: apicfg.Enabled,
+		},
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"ENABLE_HTTP2_AUTO_DETECTION": "true",
+			})
+		}),
 	}}
 
 	for _, test := range tests {
@@ -287,7 +378,16 @@ func TestMakeQueueContainer(t *testing.T) {
 					}},
 				}
 			}
-			got, err := makeQueueContainer(test.rev, &test.lc, &traceConfig, &test.oc, &test.dc)
+			cfg := &config.Config{
+				Tracing:       &traceConfig,
+				Logging:       &test.lc,
+				Observability: &test.oc,
+				Deployment:    &test.dc,
+				Config: &apicfg.Config{
+					Features: &test.fc,
+				},
+			}
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -313,6 +413,9 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 		dc   deployment.Config
 	}{{
 		name: "resources percentage in annotations",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
 			func(revision *v1.Revision) {
 				revision.Annotations = map[string]string{
@@ -338,6 +441,9 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 		}),
 	}, {
 		name: "resources percentage in annotations smaller than min allowed",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
 			func(revision *v1.Revision) {
 				revision.Annotations = map[string]string{
@@ -380,6 +486,7 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 				}}
 			}),
 		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
 			QueueSidecarCPURequest: resourcePtr(resource.MustParse("25m")),
 		},
 		want: queueContainer(func(c *corev1.Container) {
@@ -406,6 +513,7 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 				}}
 			}),
 		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
 			QueueSidecarCPURequest: resourcePtr(resource.MustParse("25m")),
 		},
 		want: queueContainer(func(c *corev1.Container) {
@@ -419,7 +527,9 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			got, err := makeQueueContainer(test.rev, &logConfig, &traceConfig, &obsConfig, &test.dc)
+			cfg := (&revCfg).DeepCopy()
+			cfg.Deployment = &test.dc
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -462,7 +572,7 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 				Scheme: corev1.URISchemeHTTP,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -479,10 +589,15 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		c.Env = env(map[string]string{
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
+		c.StartupProbe = nil // User overrode periodSeconds so no exec probe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10s"},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
 				},
 			},
 			PeriodSeconds:  1,
@@ -490,7 +605,7 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
@@ -533,7 +648,7 @@ func TestProbeGenerationHTTP(t *testing.T) {
 				Scheme: corev1.URISchemeHTTPS,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -551,17 +666,23 @@ func TestProbeGenerationHTTP(t *testing.T) {
 			"USER_PORT":               strconv.Itoa(userPort),
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
+		c.StartupProbe = nil // User overrode periodSeconds so no execprobe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10s"},
-				}},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
+				},
+			},
 			PeriodSeconds:  2,
 			TimeoutSeconds: 10,
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
@@ -609,14 +730,34 @@ func TestTCPProbeGeneration(t *testing.T) {
 			},
 		},
 		want: queueContainer(func(c *corev1.Container) {
-			c.ReadinessProbe = &corev1.Probe{
+			c.StartupProbe = &corev1.Probe{
 				Handler: corev1.Handler{
 					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "0"},
+						Command: []string{"/ko-app/queue", "-probe-timeout", "1h34m38s"},
 					},
 				},
-				PeriodSeconds:  10,
-				TimeoutSeconds: 10,
+				// StartupProbe overrides the user's parameters because the
+				// actual probing happens inside the execprobe.
+				// The timeout seconds is set to the progress deadline to match the
+				// timeout on initial deployment.
+				TimeoutSeconds:   5678,
+				PeriodSeconds:    1,
+				SuccessThreshold: 1,
+				FailureThreshold: 1,
+			}
+			c.ReadinessProbe = &corev1.Probe{
+				Handler: corev1.Handler{
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
+					},
+				},
+				PeriodSeconds:    1,
+				TimeoutSeconds:   0,
+				SuccessThreshold: 3,
 			}
 			c.Env = env(map[string]string{"USER_PORT": strconv.Itoa(userPort)})
 		}),
@@ -647,14 +788,20 @@ func TestTCPProbeGeneration(t *testing.T) {
 			TimeoutSeconds: 1,
 		},
 		want: queueContainer(func(c *corev1.Container) {
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "1s"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
-				PeriodSeconds:  1,
-				TimeoutSeconds: 1,
+				PeriodSeconds: 1,
+				// Inherit Kubernetes default here rather than overriding as we need to do for exec probe.
+				TimeoutSeconds: 0,
 			}
 			c.Env = env(map[string]string{})
 		}),
@@ -695,10 +842,15 @@ func TestTCPProbeGeneration(t *testing.T) {
 			},
 		},
 		want: queueContainer(func(c *corev1.Container) {
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "15s"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
 				PeriodSeconds:       2,
@@ -725,7 +877,7 @@ func TestTCPProbeGeneration(t *testing.T) {
 				Value: string(wantProbeJSON),
 			})
 
-			got, err := makeQueueContainer(testRev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+			got, err := makeQueueContainer(testRev, &revCfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error")
 			}
@@ -742,6 +894,7 @@ var defaultEnv = map[string]string{
 	"CONTAINER_CONCURRENCY":                 "0",
 	"ENABLE_PROFILING":                      "false",
 	"METRICS_DOMAIN":                        metrics.Domain(),
+	"METRICS_COLLECTOR_ADDRESS":             "",
 	"QUEUE_SERVING_PORT":                    "8012",
 	"REVISION_TIMEOUT_SECONDS":              "45",
 	"SERVING_CONFIGURATION":                 "",
